@@ -2,7 +2,8 @@ from cabaret.camera import Camera
 from cabaret.focuser import Focuser
 from cabaret.image import generate_image
 from cabaret.observatory import Observatory
-from cabaret.queries import Filters
+from cabaret.plot import plot_image
+from cabaret.queries import Filters, GaiaQuery
 from cabaret.site import Site
 from cabaret.sources import Sources
 from cabaret.telescope import Telescope
@@ -16,4 +17,6 @@ __all__ = [
     "Site",
     "Sources",
     "Telescope",
+    "GaiaQuery",
+    "plot_image",
 ]
