@@ -1,9 +1,10 @@
 from abc import ABC, abstractmethod
-from dataclasses import dataclass, field
+from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
 import numpy.random
+from astropy.coordinates import Angle
 from astropy.wcs import WCS
 
 
@@ -18,24 +19,21 @@ class Camera:
 
     >>> from cabaret import Camera, Observatory, Sources
     >>> pixel_defects = {
-    ...     "hot": {"type": "constant", "value": 10_000, "rate": 0.1, "seed": 0},
     ...     "cold": {"type": "constant", "value": 0, "rate": 0.01, "seed": 1},
     ...     "noise": {"type": "noise", "rate": 0.02, "noise_level": 100, "seed": 2},
-    ...     "qe": {"type": "quantum_efficiency_map", "seed": 3}
+    ...     "qe": {"type": "quantum_efficiency_map", "seed": 3},
+    ...     "smear": {"type": "readout_smear", "readout_time": 1},
     ... }
-    >>> camera = Camera(width=100, height=100, pixel_defects=pixel_defects)
+    >>> camera = Camera(width=1024, height=1024, pixel_defects=pixel_defects)
     >>> observatory = Observatory(camera=camera)
-    >>> sources = Sources.get_test_source()
-    >>> ra, dec = sources.ra.mean().deg, sources.dec.mean().deg,
+    >>> sources = Sources.get_test_sources()
+    >>> ra, dec = sources.center
 
     Now generate an image with and without pixel defects:
 
-    >>> image = observatory.generate_image(
-    ...     exp_time=10, ra=ra, dec=dec, seed=42, sources=sources
-    ... )
-    >>> clean_image = observatory.generate_image(
-    ...     exp_time=10, ra=ra, dec=dec,
-    ...     apply_pixel_defects=False, seed=42, sources=sources
+    >>> _, clean_image, image = observatory.generate_image_stack(
+    ...     ra=ra, dec=dec, exp_time=10, seed=42, sources=sources,
+    ...     convert_all_to_adu=True
     ... )
 
     To plot the images, you can use the `plot_image` function from `cabaret.plot`:
@@ -43,67 +41,106 @@ class Camera:
     >>> from cabaret.plot import plot_image
     >>> import matplotlib.pyplot as plt
     >>> fig, axes = plt.subplots(1, 2, figsize=(7, 5), sharex=True, sharey=True)
-    >>> plot_image(clean_image, ax=axes[0], title="Image without defects")
-    >>> plot_image(image, ax=axes[1], title="Image with multiple defects")
+    >>> _ = plot_image(clean_image, ax=axes[0], title="Image without defects")
+    >>> _ = plot_image(image, ax=axes[1], title="Image with multiple defects")
     >>> plt.subplots_adjust(wspace=0.1)
     >>> plt.show()
     """
 
-    name: str = "gaia-camera-simulated"
+    name: str
     """Name of the camera."""
 
-    width: int = 1024
+    width: int
     """Number of pixels in the x-direction (width)."""
 
-    height: int = 1024
+    height: int
     """Number of pixels in the y-direction (height)."""
 
-    bin_x: int = 1
+    bin_x: int
     """Binning factor in x."""
 
-    bin_y: int = 1
+    bin_y: int
     """Binning factor in y."""
 
-    pitch: float = 13.5
+    pitch: float
     """Pixel pitch in microns."""
 
-    plate_scale: float | None = None
+    plate_scale: float | None
     """Arcseconds per pixel (if None, calculated from pitch and telescope)."""
 
-    max_adu: int = 2**16 - 1
+    max_adu: int
     """Maximum ADU value."""
 
-    well_depth: int = 2**16 - 1
+    well_depth: int
     """Full well capacity in electrons."""
 
-    bias: int = 300
+    bias: int
     """Bias level in ADU."""
 
-    gain: float = 1.0
+    gain: float
     """Gain in electrons per ADU."""
 
-    read_noise: float = 6.2
+    read_noise: float
     """Read noise in electrons."""
 
-    dark_current: float = 0.2
+    dark_current: float
     """Dark current in electrons per second."""
 
-    average_quantum_efficiency: float = 0.8
+    average_quantum_efficiency: float
     """Average quantum efficiency (fraction)."""
 
-    pixel_defects: dict = field(default_factory=dict)
+    rotation: float
+    """Rotation angle of the camera in degrees."""
+
+    pixel_defects: dict
     """Dictionary of pixel defect configurations."""
 
-    def __post_init__(self):
-        if self.pixel_defects:
+    def __init__(
+        self,
+        name: str = "gaia-camera-simulated",
+        width: int = 1024,
+        height: int = 1024,
+        bin_x: int = 1,
+        bin_y: int = 1,
+        pitch: float = 13.5,
+        plate_scale: float | None = None,
+        max_adu: int = 2**16 - 1,
+        well_depth: int = 2**16 - 1,
+        bias: int = 300,
+        gain: float = 1.0,
+        read_noise: float = 6.2,
+        dark_current: float = 0.2,
+        average_quantum_efficiency: float = 0.8,
+        rotation: float = 0.0,
+        pixel_defects: dict | None = None,
+    ) -> None:
+        self.name = name
+        self.width = width
+        self.height = height
+        self.bin_x = bin_x
+        self.bin_y = bin_y
+        self.pitch = pitch
+        self.plate_scale = plate_scale
+        self.max_adu = max_adu
+        self.well_depth = well_depth
+        self.bias = bias
+        self.gain = gain
+        self.read_noise = read_noise
+        self.dark_current = dark_current
+        self.average_quantum_efficiency = average_quantum_efficiency
+        self.rotation = rotation
+
+        if pixel_defects:
             self.pixel_defects = {
                 key: (
                     self._create_pixel_defect(key, **value)
                     if isinstance(value, dict)
                     else value
                 )
-                for key, value in self.pixel_defects.items()
+                for key, value in pixel_defects.items()
             }
+        else:
+            self.pixel_defects = {}
 
     @property
     def shape(self) -> tuple[int, int]:
@@ -187,31 +224,61 @@ class Camera:
 
         return image.astype(np.uint16)
 
+    def bin_image(self, image: np.ndarray) -> np.ndarray:
+        """Bin the image according to the camera's binning factors.
+
+        Parameters
+        ----------
+        image : np.ndarray
+            The image to be binned.
+
+        Returns
+        -------
+        np.ndarray
+            The binned image.
+        """
+        if self.bin_x == 1 and self.bin_y == 1:
+            return image
+
+        binned_height = image.shape[0] // self.bin_y
+        binned_width = image.shape[1] // self.bin_x
+
+        binned_image = (
+            image[: binned_height * self.bin_y, : binned_width * self.bin_x]
+            .reshape(binned_height, self.bin_y, binned_width, self.bin_x)
+            .sum(axis=(1, 3))
+        )
+
+        return binned_image
+
     def set_plate_scale_from_focal_length(self, focal_length: float):
         """Set the plate scale based on the focal length of the telescope.
 
         Parameters
         ----------
         focal_length : float
-            Focal length of the telescope in mm.
+            Focal length of the telescope in m.
 
         Returns
         -------
         None
         """
         self.plate_scale = (
-            np.arctan((self.pitch * 1e-6) / (focal_length)) * (180 / np.pi) * 3600
+            2
+            * np.arctan((self.pitch * 1e-6) / (2 * focal_length))
+            * (180 / np.pi)
+            * 3600
         )  # "/pixel
 
     def _create_pixel_defect(
         self,
         name: str,
-        type: Literal["constant", "telegraphic", "noise", "fixed_pattern"] = "constant",
+        type: Literal["constant", "column", "noise", "fixed_pattern"] = "constant",
         **kwargs,
     ) -> "PixelDefect":
         defect_classes = {
             "constant": ConstantPixelDefect,
-            "telegraphic": TelegraphicPixelDefect,
+            "column": ColumnPixelDefect,
             "noise": RandomNoisePixelDefect,
             "quantum_efficiency_map": QuantumEfficiencyMapPixelDefect,
             "readout_smear": ReadoutSmearPixelDefect,
@@ -240,18 +307,61 @@ class Camera:
         if self.plate_scale is None:
             raise ValueError("plate_scale must be set to compute WCS.")
 
+        # Convert plate scale to degrees per pixel
+        scale_deg = self.plate_scale / 3600  # arcsec to deg
+
+        theta = np.deg2rad(self.rotation)
+        cos_theta = np.cos(theta)
+        sin_theta = np.sin(theta)
+
+        # CD matrix with rotation
+        cd11 = -scale_deg * cos_theta
+        cd12 = scale_deg * sin_theta
+        cd21 = -scale_deg * sin_theta
+        cd22 = -scale_deg * cos_theta
+
         wcs = WCS(naxis=2)
-        wcs.wcs.cdelt = [-self.plate_scale / 3600, -self.plate_scale / 3600]
+        wcs.wcs.cd = np.array([[cd11, cd12], [cd21, cd22]])
         wcs.wcs.cunit = ["deg", "deg"]
         wcs.wcs.crpix = [int(self.width / 2), int(self.height / 2)]
         wcs.wcs.crval = [center.ra.deg, center.dec.deg]
         wcs.wcs.ctype = ["RA---TAN", "DEC--TAN"]
         return wcs
 
+    def get_fov_radius(self) -> Angle:
+        """Half-diagonal field radius in degrees."""
+        if self.plate_scale is None:
+            raise ValueError("plate_scale must be set to compute FOV.")
+        fov_y = self.height * self.plate_scale / 3600.0
+        fov_x = self.width * self.plate_scale / 3600.0
+        return Angle(np.sqrt(fov_x**2 + fov_y**2) / 2, "deg")
+
     def _create_blank_image(self) -> np.ndarray:
         """Create a blank image filled with zeros for testing."""
         return np.zeros((self.height, self.width), dtype=np.uint16)
 
+    def on_camera_mask(self, pixel_coords: np.ndarray) -> np.ndarray:
+        """Clip the given pixel coordinates to the camera's bounds.
+
+        Parameters
+        ----------
+        pixel_coords : np.ndarray
+            An array of shape (n, 2) containing the pixel coordinates to be clipped.
+
+        Returns
+        -------
+        np.ndarray
+            An array of shape (m, 2) containing the pixel coordinates within the
+            camera's bounds.
+        """
+        in_camera = (
+            (pixel_coords[0, :] >= 0)
+            & (pixel_coords[0, :] < self.width)
+            & (pixel_coords[1, :] >= 0)
+            & (pixel_coords[1, :] < self.height)
+        )
+        return in_camera
+
 
 @dataclass
 class PixelDefect(ABC):
@@ -359,17 +469,11 @@ class ConstantPixelDefect(PixelDefect):
     ...     "hot": {"type": "constant", "value": 5000, "rate": 0.01, "seed": 0},
     ...     "cold": {"type": "constant", "value": 0, "rate": 0.01, "seed": 1}
     ... }
-    >>> observatory = Observatory(
-    ...     camera={"width": 80, "height": 80, "pixel_defects": pixel_defects}
-    ... )
-    >>> sources = Sources.get_test_source()
-    >>> ra, dec = sources.ra.mean().deg, sources.dec.mean().deg,
-    >>> image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources, seed=42
-    ... )
-    >>> clean_image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources,
-    ...     apply_pixel_defects=False, seed=42
+    >>> observatory = Observatory(camera={"pixel_defects": pixel_defects})
+    >>> sources = Sources.get_test_sources()
+    >>> ra, dec = sources.center
+    >>> _, clean_image, image = observatory.generate_image_stack(
+    ...     exp_time=3, ra=ra, dec=dec, sources=sources, convert_all_to_adu=True
     ... )
 
     To plot the images, you can use the `plot_image` function from `cabaret.plot`:
@@ -377,8 +481,8 @@ class ConstantPixelDefect(PixelDefect):
     >>> from cabaret.plot import plot_image
     >>> import matplotlib.pyplot as plt
     >>> fig, axes = plt.subplots(1, 2, figsize=(7, 5), sharex=True, sharey=True)
-    >>> plot_image(clean_image, ax=axes[0], title="Image without defects")
-    >>> plot_image(image, ax=axes[1], title="Image with constant defects")
+    >>> _ = plot_image(clean_image, ax=axes[0], title="Image without defects")
+    >>> _ = plot_image(image, ax=axes[1], title="Image with constant defects")
     >>> plt.subplots_adjust(wspace=0.1)
     >>> plt.show()
     """
@@ -399,36 +503,31 @@ class ConstantPixelDefect(PixelDefect):
 
 
 @dataclass
-class TelegraphicPixelDefect(PixelDefect):
-    """A pixel defect that simulates telegraphic noise by selecting entire rows or
+class ColumnPixelDefect(PixelDefect):
+    """A pixel defect that simulates column noise by selecting entire rows or
     columns of pixels to be defective.
 
     Examples
     --------
     >>> from cabaret import Observatory, Sources
     >>> pixel_defects = {
-    ...     "telegraphic": {"type": "telegraphic", "value": 5000, "rate": 0.5, "dim": 0}
+    ...     "column": {"type": "column", "value": 20, "rate": 0.01, "dim": 1}
     ... }
-    >>> observatory = Observatory(
-    ...     camera={"width": 80, "height": 80, "pixel_defects": pixel_defects}
-    ... )
-    >>> sources = Sources.get_test_source()
-    >>> ra, dec = sources.ra.mean().deg, sources.dec.mean().deg,
-    >>> image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources, seed=42
-    ... )
-    >>> clean_image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources,
-    ...     apply_pixel_defects=False, seed=42
+    >>> observatory = Observatory(camera={"pixel_defects": pixel_defects})
+    >>> sources = Sources.get_test_sources()
+    >>> ra, dec = sources.center
+    >>> _, clean_image, image = observatory.generate_image_stack(
+    ...     exp_time=3, ra=ra, dec=dec, sources=sources, convert_all_to_adu=True
     ... )
 
+
     To plot the images, you can use the `plot_image` function from `cabaret.plot`:
 
     >>> from cabaret.plot import plot_image
     >>> import matplotlib.pyplot as plt
     >>> fig, axes = plt.subplots(1, 2, figsize=(7, 5), sharex=True, sharey=True)
-    >>> plot_image(clean_image, ax=axes[0], title="Image without defects")
-    >>> plot_image(image, ax=axes[1], title="Image with telegraphic defects")
+    >>> _ = plot_image(clean_image, ax=axes[0], title="Image without defects")
+    >>> _ = plot_image(image, ax=axes[1], title="Image with column defects")
     >>> plt.subplots_adjust(wspace=0.1)
     >>> plt.show()
     """
@@ -494,26 +593,22 @@ class RandomNoisePixelDefect(PixelDefect):
     Examples
     --------
     >>> from cabaret import Observatory, Sources
-    >>> pixel_defects = {"noise": {"type": "noise", "rate": 0.01, "noise_level": 5e3}}
-    >>> observatory = Observatory(
-    ...     camera={"width": 80, "height": 80, "pixel_defects": pixel_defects}
-    ... )
-    >>> sources = Sources.get_test_source()
-    >>> ra, dec = sources.ra.mean().deg, sources.dec.mean().deg,
-    >>> image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources
-    ... )
-    >>> clean_image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources, apply_pixel_defects=False
+    >>> pixel_defects = {"noise": {"type": "noise", "rate": 0.1, "noise_level": 5e3}}
+    >>> observatory = Observatory(camera={"pixel_defects": pixel_defects})
+    >>> sources = Sources.get_test_sources()
+    >>> ra, dec = sources.center
+    >>> _, clean_image, image = observatory.generate_image_stack(
+    ...     exp_time=3, ra=ra, dec=dec, sources=sources, convert_all_to_adu=True
     ... )
 
+
     To plot the images, you can use the `plot_image` function from `cabaret.plot`:
 
     >>> from cabaret.plot import plot_image
     >>> import matplotlib.pyplot as plt
     >>> fig, axes = plt.subplots(1, 2, figsize=(7, 5), sharex=True, sharey=True)
-    >>> plot_image(clean_image, ax=axes[0], title="Image without defects")
-    >>> plot_image(image, ax=axes[1], title="Image with random noise defects")
+    >>> _ = plot_image(clean_image, ax=axes[0], title="Image without defects")
+    >>> _ = plot_image(image, ax=axes[1], title="Image with random noise defects")
     >>> plt.subplots_adjust(wspace=0.1)
     >>> plt.show()
     """
@@ -558,18 +653,13 @@ class QuantumEfficiencyMapPixelDefect(PixelDefect):
     --------
     >>> from cabaret import Observatory, Sources
     >>> pixel_defects = {
-    ...     "qe": {"type": "quantum_efficiency_map", "quantum_efficiency_std": 0.25}
+    ...     "qe": {"type": "quantum_efficiency_map", "quantum_efficiency_std": 0.5}
     ... }
-    >>> observatory = Observatory(
-    ...     camera={"width": 40, "height": 40, "pixel_defects": pixel_defects}
-    ... )
-    >>> sources = Sources.get_test_source()
-    >>> ra, dec = sources.ra.mean().deg, sources.dec.mean().deg,
-    >>> image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources
-    ... )
-    >>> clean_image = observatory.generate_image(
-    ...     exp_time=3, ra=ra, dec=dec, sources=sources, apply_pixel_defects=False
+    >>> observatory = Observatory(camera={"pixel_defects": pixel_defects})
+    >>> sources = Sources.get_test_sources()
+    >>> ra, dec = sources.center
+    >>> _, clean_image, image = observatory.generate_image_stack(
+    ...     exp_time=3, ra=ra, dec=dec, sources=sources, convert_all_to_adu=True
     ... )
 
     To plot the images, you can use the `plot_image` function from `cabaret.plot`:
@@ -577,9 +667,9 @@ class QuantumEfficiencyMapPixelDefect(PixelDefect):
     >>> from cabaret.plot import plot_image
     >>> import matplotlib.pyplot as plt
     >>> fig, axes = plt.subplots(1, 3, figsize=(10, 5), sharex=True, sharey=True)
-    >>> plot_image(clean_image, ax=axes[0], title="Image without defects")
-    >>> plot_image(image, ax=axes[1], title="Image with QE defects")
-    >>> plot_image(
+    >>> _ = plot_image(clean_image, ax=axes[0], title="Image without defects")
+    >>> _ = plot_image(image, ax=axes[1], title="Image with QE defects")
+    >>> _ = plot_image(
     ...     observatory.camera.pixel_defects["qe"].quantum_efficiency_map,
     ...     ax=axes[2], title="Quantum Efficiency Map"
     ... )
@@ -641,28 +731,22 @@ class ReadoutSmearPixelDefect(PixelDefect):
     Examples
     --------
     >>> from cabaret import Observatory, Sources
-    >>> pixel_defects = {
-    ...     "smear": {"type": "readout_smear", "readout_time": 1}
-    ... }
-    >>> observatory = Observatory(
-    ...     camera={"width": 40, "height": 40, "pixel_defects": pixel_defects}
-    ... )
-    >>> sources = Sources.get_test_source()
-    >>> ra, dec = sources.ra.mean().deg, sources.dec.mean().deg,
-    >>> image = observatory.generate_image(
-    ...     exp_time=1, ra=ra, dec=dec, sources=sources
-    ... )
-    >>> clean_image = observatory.generate_image(
-    ...     exp_time=1, ra=ra, dec=dec, sources=sources, apply_pixel_defects=False
+    >>> pixel_defects = {"smear": {"type": "readout_smear", "readout_time": 1}}
+    >>> observatory = Observatory(camera={"pixel_defects": pixel_defects})
+    >>> sources = Sources.get_test_sources()
+    >>> ra, dec = sources.center
+    >>> _, clean_image, image = observatory.generate_image_stack(
+    ...     exp_time=5, ra=ra, dec=dec, sources=sources, convert_all_to_adu=True
     ... )
 
+
     To plot the images, you can use the `plot_image` function from `cabaret.plot`:
 
     >>> from cabaret.plot import plot_image
     >>> import matplotlib.pyplot as plt
     >>> fig, axes = plt.subplots(1, 2, figsize=(7, 5), sharex=True, sharey=True)
-    >>> plot_image(clean_image, ax=axes[0], title="Image without defects")
-    >>> plot_image(image, ax=axes[1], title="Image with readout smear")
+    >>> _ = plot_image(clean_image, ax=axes[0], title="Image without defects")
+    >>> _ = plot_image(image, ax=axes[1], title="Image with readout smear")
     >>> plt.subplots_adjust(wspace=0.1)
     >>> plt.show()
     """
