@@ -4,12 +4,13 @@ from datetime import UTC, datetime
 import numpy as np
 import numpy.random
 from astropy import units as u
-from astropy.coordinates import AltAz, EarthLocation, SkyCoord, get_sun
+from astropy.coordinates import AltAz, Angle, EarthLocation, SkyCoord, get_sun
 from astropy.time import Time
+from astropy.wcs import WCS
 
 from cabaret.camera import Camera
 from cabaret.focuser import Focuser
-from cabaret.queries import Filters, get_gaia_sources
+from cabaret.queries import Filters, GaiaQuery
 from cabaret.site import Site
 from cabaret.sources import Sources
 from cabaret.telescope import Telescope
@@ -59,46 +60,55 @@ def moffat_profile(
     return mp / mp_sum
 
 
-def generate_star_image_slow(
-    pos: np.ndarray,
-    fluxes: list[float],
-    FWHM: float,
-    frame_size: tuple[int, int],
-    rng: numpy.random.Generator,
-) -> np.ndarray:
+def scintillation_noise(
+    r: float,
+    t: float,
+    N_star: float,
+    h: float = 2440,
+    C: float = 1.56,
+    airmass: float = 1.5,
+) -> float:
     """
-    Render stars onto an image using a slow loop-based approach.
+    Calculate the scintillation noise for a given set of parameters.
 
     Parameters
     ----------
-    pos : np.ndarray
-        Pixel positions of stars (shape: 2, n_stars).
-    fluxes : list
-        list of fluxes for each star.
-    FWHM : float
-        Full width at half maximum for the Moffat profile.
-    frame_size : tuple
-        Size of the output image (width, height).
-    rng : numpy.random.Generator
-        Random number generator for Poisson noise.
+    r : float
+        Aperture radius in meters.
+    t : float
+        Exposure time in seconds.
+    N_star : float
+        Number of stars.
+    h : float
+        Altitude of the observatory in meters. Default is 2440 for Paranal Observatory.
+    C : float
+        Empirical coefficient. Default is 1.56, optimized for the
+        20-cm NGTS telescopes at Paranal Observatory.
+    airmass : float, optional
+        Airmass value. Default is 1.5.
 
     Returns
     -------
-    np.ndarray
-        Image with rendered stars.
+    float
+        The calculated scintillation noise.
+
+    Reference
+    ---------
+    https://academic.oup.com/mnras/article/509/4/6111/6442285
     """
-    x = np.linspace(0, frame_size[0] - 1, frame_size[0])
-    y = np.linspace(0, frame_size[1] - 1, frame_size[1])
-    xx, yy = np.meshgrid(x, y)
 
-    image = np.zeros(frame_size).T
-    for i, flux in enumerate(fluxes):
-        x0 = pos[0][i]
-        y0 = pos[1][i]
-        star = rng.poisson(flux) * moffat_profile(xx, yy, x0, y0, FWHM)
-        image += star
-
-    return image
+    return (
+        np.sqrt(
+            1e-5
+            * C**2
+            * pow(2 * r, -4 / 3)
+            * t**-1
+            * airmass**3
+            * np.exp(-2 * h / 8000)
+        )
+        * N_star
+        * t
+    )
 
 
 def generate_star_image(
@@ -107,6 +117,11 @@ def generate_star_image(
     FWHM: float,
     frame_size: tuple[int, int],
     rng: numpy.random.Generator,
+    telescope_aperture: float,
+    site_elevation: float,
+    exp_time: float,
+    airmass: float = 1.5,
+    fwhm_multiplier: float = 5.0,
 ) -> np.ndarray:
     """
     Render stars onto an image using a fast, windowed approach.
@@ -123,6 +138,8 @@ def generate_star_image(
         Size of the output image (width, height).
     rng : numpy.random.Generator
         Random number generator for Poisson noise.
+    fwhm_multiplier : float, optional
+        Multiplier to determine the rendering radius around each star (default: 5.0).
 
     Returns
     -------
@@ -133,7 +150,7 @@ def generate_star_image(
     y = np.linspace(0, frame_size[1] - 1, frame_size[1])
     xx, yy = np.meshgrid(x, y)
 
-    render_radius = FWHM * 5  # render 5 FWHM around the star
+    render_radius = FWHM * fwhm_multiplier  # render n * FWHM around the star
 
     image = np.zeros(frame_size).T
     for i, flux in enumerate(fluxes):
@@ -147,13 +164,28 @@ def generate_star_image(
         x_min, x_max = max(0, x_min), min(x_max, frame_size[0] - 1)
         y_min, y_max = max(0, y_min), min(y_max, frame_size[1] - 1)
 
-        star = rng.poisson(flux) * moffat_profile(
+        scint_noise = scintillation_noise(
+            r=telescope_aperture / 2,
+            t=exp_time,
+            N_star=flux,
+            h=site_elevation,
+            C=1.56,
+            airmass=airmass,
+        )
+
+        star_flux = rng.poisson(flux) + rng.normal(0, scint_noise)
+
+        if star_flux < 0:
+            star_flux = 0
+
+        star = star_flux * moffat_profile(
             xx[y_min : y_max + 1, x_min : x_max + 1],
             yy[y_min : y_max + 1, x_min : x_max + 1],
             x0,
             y0,
             FWHM,
         )
+
         image[y_min : y_max + 1, x_min : x_max + 1] += star
 
     return image
@@ -161,8 +193,7 @@ def generate_star_image(
 
 def get_sources(
     center: SkyCoord,
-    fovx: float,
-    fovy: float,
+    radius: Angle,
     dateobs: datetime,
     n_star_limit: int,
     filter_band: Filters | str,
@@ -171,14 +202,16 @@ def get_sources(
 ) -> Sources:
     """Get sources from Gaia or use provided sources."""
     if not isinstance(sources, Sources):
-        sources = get_gaia_sources(
+        logger.info("Querying Gaia for sources...")
+        sources = GaiaQuery.get_sources(
             center=center,
-            fov=u.Quantity((fovx * 1.5, fovy * 1.5), "deg"),
+            radius=radius,
             dateobs=dateobs,
             limit=n_star_limit,
             filter_band=filter_band,
             timeout=timeout,
         )
+        logger.info(f"Found {len(sources)} sources (user set limit of {n_star_limit}).")
     return sources
 
 
@@ -192,41 +225,84 @@ def add_sun_sky_background(
     logger: logging.Logger,
 ) -> np.ndarray:
     """Add sky background and sunlight if location is specified."""
-    if site.latitude is not None and site.longitude is not None:
-        logger.info(
-            "Since location specified, calculating sunlight brightness"
-            " based on sun's position"
-        )
-        location = EarthLocation(
-            lat=u.Quantity(site.latitude, "deg"), lon=u.Quantity(site.longitude, "deg")
-        )
-        obs_time = Time(dateobs, scale="utc")
-        sun = get_sun(obs_time)
-        altaz_frame = AltAz(obstime=obs_time, location=location)
-        sun_altaz = sun.transform_to(altaz_frame)
-        sun_altitude: float = sun_altaz.alt.degree  # type: ignore
-        logger.info(f"Sun altitude: {sun_altitude:.5f} deg at {dateobs} UTC")
 
-        a, b, c = (
-            np.float64(4533508.655833181),
-            np.float64(0.3937301435229289),
-            np.float64(-0.7907223506021084),
-        )  # calibrated for I+z band in Paranal
+    # Sky brightness model due to sunlight scattering
+    # calibrated for I+z band in Paranal
+    SKY_MODEL_A = np.float64(4533508.655833181)
+    SKY_MODEL_B = np.float64(0.3937301435229289)
+    SKY_MODEL_C = np.float64(-0.7907223506021084)
 
-        sky_brightness = a * b ** (c * sun_altitude)  # e-/m2/arcsec2/s
-        logger.info(f"sky_brightness (e-/m2/arcsec2/s): {sky_brightness}")
+    sun_altitude = _get_sun_altitude(site, dateobs, logger)
 
-        sky_e = (
-            sky_brightness * telescope.collecting_area * camera.plate_scale**2
-        )  # e-/s
-        logger.info(f"sky_e (e-/s): {sky_e}")
+    if sun_altitude is None:
+        return image
+
+    sun_brightness = SKY_MODEL_A * SKY_MODEL_B ** (SKY_MODEL_C * sun_altitude)
+    logger.debug(f"sun_brightness (ph/m2/arcsec2/s): {sun_brightness}")
+
+    sun_electrons_per_sec = (
+        sun_brightness
+        * telescope.collecting_area
+        * camera.average_quantum_efficiency
+        * camera.plate_scale**2
+    )
+    logger.debug(f"sun_e (e-/s): {sun_electrons_per_sec}")
+
+    sun_electrons = sun_electrons_per_sec * exp_time
+    sun_background = np.random.poisson(
+        np.ones((camera.height, camera.width), dtype=np.float64) * sun_electrons
+    ).astype(np.float64)
+
+    image += sun_background
 
-        image += np.random.poisson(
-            np.ones((camera.height, camera.width)).astype(np.float64) * sky_e * exp_time
-        ).astype(np.float64)
     return image
 
 
+def _get_sun_altitude(
+    site: Site,
+    dateobs: datetime,
+    logger: logging.Logger,
+) -> float | None:
+    """
+    Calculate the sun's altitude angle at the observation time.
+
+    Parameters
+    ----------
+    site : Site
+        Observatory site configuration.
+    dateobs : datetime
+        Observation date and time.
+    logger : logging.Logger
+        Logger instance for debug messages.
+
+    Returns
+    -------
+    float or None
+        Sun altitude in degrees, or None if it cannot be calculated.
+    """
+    if site.sun_altitude is not None:
+        return site.sun_altitude
+
+    if site.latitude is None or site.longitude is None:
+        return None
+
+    logger.debug("Calculating sunlight brightness based on sun's position")
+
+    location = EarthLocation(
+        lat=u.Quantity(site.latitude, "deg"),
+        lon=u.Quantity(site.longitude, "deg"),
+    )
+    obs_time = Time(dateobs, scale="utc")
+    sun = get_sun(obs_time)
+    altaz_frame = AltAz(obstime=obs_time, location=location)
+    sun_altaz = sun.transform_to(altaz_frame)
+    sun_altitude: float = sun_altaz.alt.degree  # type: ignore
+
+    logger.debug(f"Sun altitude: {sun_altitude:.5f} deg at {dateobs} UTC")
+
+    return sun_altitude
+
+
 def add_stars(
     image: np.ndarray,
     sources: Sources,
@@ -238,6 +314,10 @@ def add_stars(
     rng: numpy.random.Generator,
     ra: float | None,
     dec: float | None,
+    wcs: WCS | None,
+    dateobs: datetime | None,
+    airmass: float | None,
+    fwhm_multiplier: float = 5.0,
 ) -> np.ndarray:
     """Add stars to the image using the Moffat profile and sky background."""
     if len(sources) > 0:
@@ -249,17 +329,51 @@ def add_stars(
         )  # [electrons]
 
         if ra is None or dec is None:
-            ra, dec = sources.ra.deg.mean(), sources.dec.deg.mean()
+            ra, dec = sources.center
 
-        wcs = camera.get_wcs(SkyCoord(ra=ra, dec=dec, unit="deg"))
+        if wcs is None:
+            wcs = camera.get_wcs(SkyCoord(ra=ra, dec=dec, unit="deg"))
         gaias_pixel = sources.to_pixel(wcs)
+        on_camera_mask = camera.on_camera_mask(gaias_pixel)
+        logger.debug(
+            f"Number of stars on camera: {on_camera_mask.sum()}"
+            f" out of {len(sources)} total stars "
+            f"({on_camera_mask.sum() / len(sources) * 100})%."
+        )
+
+        if airmass is not None:
+            logger.debug(f"Using provided airmass: {airmass:.3f}.")
+        elif site.latitude and site.longitude:
+            location = EarthLocation(
+                lat=u.Quantity(site.latitude, "deg"),
+                lon=u.Quantity(site.longitude, "deg"),
+            )
+            altitude_azimuth_frame = AltAz(obstime=Time(dateobs), location=location)
+            center_coord = SkyCoord(ra=ra, dec=dec, unit="deg")
+            center_altaz = center_coord.transform_to(altitude_azimuth_frame)
+            zenith_angle = 90.0 - center_altaz.alt.degree  # degrees
+            airmass = 1.0 / np.cos(np.radians(zenith_angle))
+
+            if airmass < 1.0:
+                logger.warning(
+                    f"Calculated airmass {airmass:.3f} is less than 1.0."
+                    " Setting airmass to 1.0."
+                )
+                airmass = 1.0  # minimum airmass is 1.0
+        else:
+            airmass = 1.5  # default value
 
         stars = generate_star_image(
-            gaias_pixel,
-            fluxes,
+            gaias_pixel[:, on_camera_mask],
+            fluxes[on_camera_mask],
             focuser.seeing_multiplier * site.seeing / camera.plate_scale,
             (camera.width, camera.height),
             rng=rng,
+            telescope_aperture=telescope.diameter,
+            site_elevation=site.elevation if site.elevation is not None else 0,
+            exp_time=exp_time,
+            airmass=airmass,
+            fwhm_multiplier=fwhm_multiplier,
         ).astype(np.float64)
 
         sky_background = (
@@ -278,43 +392,46 @@ def add_stars(
 
 def add_stars_and_sky(
     base: np.ndarray,
-    ra: float,
-    dec: float,
+    ra: float | None,
+    dec: float | None,
     exp_time: float,
-    dateobs: datetime,
+    dateobs: datetime | None,
     light: int,
     camera: Camera,
     focuser: Focuser,
     telescope: Telescope,
     site: Site,
     filter_band: Filters | str,
+    airmass: float,
     n_star_limit: int,
     rng: numpy.random.Generator,
     timeout: float | None,
     sources: Sources | None,
+    wcs: WCS | None,
+    fwhm_multiplier: float = 5.0,
 ) -> np.ndarray:
     """Add stars and sky background to the base image."""
     if light == 1:
+        if ra is None and dec is None and not isinstance(sources, Sources):
+            raise ValueError("Either ra/dec or sources must be provided for light.")
+        if ra is None or dec is None and isinstance(sources, Sources):
+            ra, dec = sources.center  # type: ignore
         center = SkyCoord(ra=ra, dec=dec, unit="deg")
-        fovx = (
-            (1 / np.abs(np.cos(center.dec.rad)))  # type: ignore
-            * camera.width
-            * camera.plate_scale
-            / 3600
-        )
-        fovy = np.sqrt(2) * camera.height * camera.plate_scale / 3600
-        logger.info("Querying Gaia for sources...")
+
+        assert camera.plate_scale is not None, "Camera plate scale must be set."
+        radius = camera.get_fov_radius()
+
+        if dateobs is None:
+            dateobs = datetime.now(UTC)
         sources = get_sources(
             center=center,
-            fovx=fovx,
-            fovy=fovy,
+            radius=radius,
             dateobs=dateobs,
             n_star_limit=n_star_limit,
             filter_band=filter_band,
             timeout=timeout,
             sources=sources,
         )
-        logger.info(f"Found {len(sources)} sources (user set limit of {n_star_limit}).")
         image = base
         image = add_sun_sky_background(
             image, site, telescope, camera, exp_time, dateobs, logger
@@ -330,6 +447,10 @@ def add_stars_and_sky(
             rng=rng,
             ra=ra,
             dec=dec,
+            wcs=wcs,
+            dateobs=dateobs,
+            airmass=airmass,
+            fwhm_multiplier=fwhm_multiplier,
         )
     else:
         image = base
@@ -340,19 +461,21 @@ def generate_image(
     ra: float,
     dec: float,
     exp_time: float,
-    dateobs: datetime = datetime.now(UTC),
+    dateobs: datetime | None = None,
     light: int = 1,
     camera: Camera = Camera(),
     focuser: Focuser = Focuser(),
     telescope: Telescope = Telescope(),
     site: Site = Site(),
     filter_band: Filters | str = Filters.G,
+    airmass: float = 1.5,
     n_star_limit: int = 2000,
     rng: numpy.random.Generator = numpy.random.default_rng(),
     seed: int | None = None,
     timeout: float | None = None,
     sources: Sources | None = None,
-    apply_pixel_defects: bool = True,
+    wcs: WCS | None = None,
+    fwhm_multiplier: float = 5.0,
 ) -> np.ndarray:
     """
     Generate a simulated astronomical image.
@@ -379,6 +502,8 @@ def generate_image(
         Observatory site configuration.
     filter_band : Filters or str, optional
         The filter to use for the flux column. Default is "G".
+    airmass : float, optional
+        Airmass at the image center (default: 1.5).
     n_star_limit : int, optional
         Maximum number of stars to simulate.
     rng : numpy.random.Generator, optional
@@ -389,8 +514,10 @@ def generate_image(
         Timeout for Gaia query.
     sources : Sources or None, optional
         Precomputed sources to use instead of querying Gaia.
-    apply_pixel_defects : bool, optional
-        Whether to apply pixel defects to the image.
+    wcs : WCS or None, optional
+        World Coordinate System information for the image.
+    fwhm_multiplier : float, optional
+        Multiplier to determine the rendering radius around each star (default: 5.0).
 
     Returns
     -------
@@ -418,16 +545,20 @@ def generate_image(
             telescope=telescope,
             site=site,
             filter_band=filter_band,
+            airmass=airmass,
             n_star_limit=n_star_limit,
             rng=rng,
             timeout=timeout,
             sources=sources,
+            wcs=wcs,
+            fwhm_multiplier=fwhm_multiplier,
         )
     else:
         image = base
 
-    if apply_pixel_defects:
-        image = camera.apply_pixel_defects(image, exp_time)
+    image = camera.apply_pixel_defects(image, exp_time)
+
+    image = camera.bin_image(image)
 
     image = camera.to_adu_image(image)
 
@@ -438,19 +569,22 @@ def generate_image_stack(
     ra: float,
     dec: float,
     exp_time: float,
-    dateobs: datetime = datetime.now(UTC),
+    dateobs: datetime | None = None,
     light: int = 1,
     camera: Camera = Camera(),
     focuser: Focuser = Focuser(),
     telescope: Telescope = Telescope(),
     site: Site = Site(),
     filter_band: Filters | str = Filters.G,
+    airmass: float = 1.5,
     n_star_limit: int = 2000,
     rng: numpy.random.Generator = numpy.random.default_rng(),
     seed: int | None = None,
     timeout: float | None = None,
     sources: Sources | None = None,
-    apply_pixel_defects: bool = True,
+    convert_all_to_adu: bool = False,
+    wcs: WCS | None = None,
+    fwhm_multiplier: float = 5.0,
 ) -> np.ndarray:
     """
     Generate a stack of images from different stages in the image simulation pipeline.
@@ -477,6 +611,8 @@ def generate_image_stack(
         Observatory site configuration.
     filter_band : Filters or str, optional
         The filter to use for the flux column. Default is "G".
+    airmass : float, optional
+        Airmass at the image center (default: 1.5).
     n_star_limit : int, optional
         Maximum number of stars to simulate.
     rng : numpy.random.Generator, optional
@@ -487,8 +623,12 @@ def generate_image_stack(
         Timeout for Gaia query.
     sources : Sources or None, optional
         Precomputed sources to use instead of querying Gaia.
-    apply_pixel_defects : bool, optional
-        Whether to apply pixel defects to the image.
+    convert_all_to_adu : bool, optional
+        Whether to convert all images to ADU. Default is False.
+    wcs : WCS or None, optional
+        World Coordinate System information for the image.
+    fwhm_multiplier : float, optional
+        Multiplier to determine the rendering radius around each star (default: 5.0).
 
     Returns
     -------
@@ -520,19 +660,25 @@ def generate_image_stack(
             telescope=telescope,
             site=site,
             filter_band=filter_band,
+            airmass=airmass,
             n_star_limit=n_star_limit,
             rng=rng,
             timeout=timeout,
             sources=sources,
+            wcs=wcs,
+            fwhm_multiplier=fwhm_multiplier,
         )
     else:
         image = base
 
-    if apply_pixel_defects:
-        adu_image = camera.apply_pixel_defects(image.copy(), exp_time)
+    adu_image = camera.apply_pixel_defects(image.copy(), exp_time)
 
     adu_image = camera.to_adu_image(adu_image)
 
+    if convert_all_to_adu:
+        base = camera.to_adu_image(base)
+        image = camera.to_adu_image(image)
+
     return np.stack([base, image, adu_image])
 
 
@@ -548,8 +694,8 @@ if __name__ == "__main__":
 
     # example usage
     image = generate_image(
-        323.36152,
-        -0.82325,
+        ra=323.36152,
+        dec=-0.82325,
         exp_time=exp_time,
         camera=camera,
         telescope=telescope,
