@@ -18,7 +18,9 @@ class Site:
 
     longitude: float | None = None
     """Site longitude in degrees."""
-    sky_background: float = 150
-    seeing: float = 1.3
-    latitude: float | None = None
-    longitude: float | None = None
+
+    elevation: float | None = None
+    """Site elevation in meters."""
+
+    sun_altitude: float | None = None
+    """Sun altitude in degrees."""
